(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     21913,        531]*)
(*NotebookOutlinePosition[     22557,        553]*)
(*  CellTagsIndexPosition[     22513,        549]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Solves the LP problem defined by:
min(c) s.t.
A.x = b
lb <= x <= ub\
\>", "Subtitle"],

Cell[BoxData[{
    \(\(rootDir\  = \ \
"\<C:\\DATADIR\Programmazione\java\joptimizer\\src\\test\\resources\\lp\\\
presolving\\\>";\)\), "\[IndentingNewLine]", 
    \(\(problemId\  = \ "\<8\>";\)\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        StyleBox["c",
          "MR"], 
        StyleBox["=",
          "MR"], 
        RowBox[{
          StyleBox["Import",
            "MR"], 
          StyleBox["[",
            "MR"], 
          RowBox[{
            RowBox[{"rootDir", " ", "<>", 
              StyleBox["\"\<c\>\"",
                "MR"], 
              StyleBox[" ",
                "MR"], 
              StyleBox["<>",
                "MR"], "problemId", "<>", " ", 
              StyleBox["\"\<.txt\>\"",
                "MR"]}], 
            StyleBox[",",
              "MR"], " ", Cell[BoxData[
                FormBox[
                  StyleBox["\"\<List\>\"",
                    "MR"], TraditionalForm]]]}], 
          StyleBox["]",
            "MR"]}]}], 
      StyleBox[";",
        "MR"]}], "\[IndentingNewLine]", \(n = 
        Length[c];\), "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"A", "=", 
        RowBox[{"ReadList", "[", 
          RowBox[{
            RowBox[{"rootDir", " ", "<>", " ", 
              StyleBox["\"\<A\>\"",
                "MR"], 
              StyleBox["<>",
                "MR"], 
              StyleBox["problemId",
                "MR"], 
              StyleBox["<>",
                "MR"], 
              StyleBox["\"\<.txt\>\"",
                "MR"]}], ",", \(Table[Number, {n}]\)}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        StyleBox["b",
          "MR"], 
        StyleBox["=",
          "MR"], 
        RowBox[{"ReadList", "[", 
          RowBox[{
            RowBox[{"rootDir", " ", "<>", " ", 
              StyleBox["\"\<b\>\"",
                "MR"], 
              StyleBox["<>",
                "MR"], 
              StyleBox["problemId",
                "MR"], 
              StyleBox["<>",
                "MR"], 
              StyleBox["\"\<.txt\>\"",
                "MR"]}], ",", "Number"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", \(bCoeff\  = \ 
        Table[{b[\([i]\)], 0}, \ {i, Length[b]}];\), "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        StyleBox["lb",
          "MR"], 
        StyleBox["=",
          "MR"], 
        RowBox[{
          RowBox[{
            StyleBox["Import",
              "MR"], 
            StyleBox["[",
              "MR"], 
            RowBox[{
              RowBox[{"rootDir", " ", "<>", " ", 
                StyleBox["\"\<lb\>\"",
                  "MR"], 
                StyleBox["<>",
                  "MR"], 
                StyleBox["problemId",
                  "MR"], 
                StyleBox["<>",
                  "MR"], 
                StyleBox["\"\<.txt\>\"",
                  "MR"]}], 
              StyleBox[",",
                "MR"], " ", Cell[BoxData[
                  FormBox[
                    StyleBox["\"\<List\>\"",
                      "MR"], TraditionalForm]]]}], 
            StyleBox["]",
              "MR"]}], "/.", 
          RowBox[{"{", 
            RowBox[{"\"\<NaN\>\"", "\[Rule]", 
              StyleBox[\(-999999\),
                "MR"]}], "}"}]}]}], 
      StyleBox[";",
        "MR"]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        StyleBox["ub",
          "MR"], 
        StyleBox["=",
          "MR"], 
        RowBox[{
          RowBox[{
            StyleBox["Import",
              "MR"], 
            StyleBox["[",
              "MR"], 
            RowBox[{
              RowBox[{"rootDir", " ", "<>", " ", 
                StyleBox["\"\<ub\>\"",
                  "MR"], 
                StyleBox["<>",
                  "MR"], 
                StyleBox["problemId",
                  "MR"], 
                StyleBox["<>",
                  "MR"], 
                StyleBox["\"\<.txt\>\"",
                  "MR"]}], 
              StyleBox[",",
                "MR"], " ", Cell[BoxData[
                  FormBox[
                    StyleBox["\"\<List\>\"",
                      "MR"], TraditionalForm]]]}], 
            StyleBox["]",
              "MR"]}], "/.", 
          RowBox[{"{", 
            RowBox[{"\"\<NaN\>\"", "\[Rule]", 
              RowBox[{"+", 
                StyleBox["999999",
                  "MR"]}]}], "}"}]}]}], 
      ";"}], "\[IndentingNewLine]", \(variablesBounds\  = \ 
        Table[{lb[\([i]\)], \ ub[\([i]\)]}, \ {i, 
            n}];\), "\[IndentingNewLine]", "n"}], "Input"],

Cell[BoxData[
    \(466\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(mySol = LinearProgramming[c, A, bCoeff, \ variablesBounds]\)], "Input"],

Cell[BoxData[
    \({8.870754920442141`*^-18, 0.1299999999952562`, 0.1499999999961636`, 
      0.04999999999756033`, 0.039999999997352716`, 0.06999999998068805`, 
      0.11999999999891907`, 0.3999999999963522`, 0.2499999999991022`, 
      0.061999999987409515`, 0.5199999999897953`, 1.01866694449008`*^-12, 
      0.0033999999945788636`, 0.06799999999048587`, 1.4439999999472173`, 
      1.5727395218457315`*^-17, 0.0019999999992070428`, 
      0.008499999945629106`, 3.5085704631530495`*^-18, 0.11999999998181793`, 
      0.15999999998408743`, 0.05499999999069458`, 0.039999999986569904`, 
      0.07999999990197608`, 0.1289999999971338`, 0.4199999999860834`, 
      0.28999999999782977`, 0.07299999993701549`, 0.4699999999476446`, 
      1.1040947245543038`*^-12, 0.0033999999747221115`, 0.04589999994143922`, 
      1.2349999997461676`, 1.4112710737475346`*^-17, 0.00599999999851582`, 
      0.006999999747149774`, 7.149608002135801`*^-18, 0.04999999999622693`, 
      0.17999999999489835`, 0.05999999999673262`, 0.04999999999634284`, 
      0.08999999998874297`, 0.14679999999610666`, 0.4499999999836044`, 
      0.3299999999972626`, 0.08699999998523038`, 0.38999999999091245`, 
      3.0366576533705263`*^-12, 0.0033999999943468773`, 0.02549999998556275`, 
      0.8549989999393681`, 1.0450322710260169`*^-17, 0.01799999999981647`, 
      0.0019999999292639775`, 1.6203789539424995`*^-17, 
      0.039999999999196525`, 0.1999999999988875`, 0.06499999999930423`, 
      0.049999999999418475`, 0.09999999999742289`, 0.1679999999991901`, 
      0.4999999999980203`, 0.40999999999912046`, 0.10999999999758868`, 
      0.2000000000054046`, 4.081522750008681`*^-12, 0.0034000000001749636`, 
      2.9549447604508517`*^-12, 0.09500000000076649`, 
      7.103359193062193`*^-18, 0.029999999989023553`, 
      7.220516713040696`*^-18, 2.2190051569850224`*^-17, 
      0.03999999999910192`, 0.19999999999854845`, 0.06999999999921709`, 
      0.05999999999925876`, 0.10999999999754967`, 0.24499999999839397`, 
      0.5499999999972517`, 0.49999999999819056`, 0.14999999999813574`, 
      1.057138182466969`*^-11, 5.822800985865235`*^-12, 
      1.6933212848841765`*^-12, 2.917292080436531`*^-12, 
      9.250885606047376`*^-12, 1.0989798783503755`*^-17, 
      0.009999999984778206`, 1.1052585329627482`*^-17, 
      1.3981458486435474`*^-17, 8.418990029731015`*^-18, 0.1999999999992333`, 
      0.07999999999875332`, 0.06999999999916477`, 0.11999999999760437`, 
      0.288999999998232`, 0.5999999999977351`, 0.6799999999970756`, 
      0.19999999999761667`, 2.8650293553621832`*^-11, 
      1.1192817463577199`*^-11, 5.223489966907135`*^-12, 
      8.107520411815178`*^-12, 3.933831276984045`*^-11, 
      6.29967176200621`*^-18, 6.311104966563831`*^-18, 
      6.30206333392473`*^-18, 5.437092170247548`*^-11, 
      7.929832037699737`*^-13, 1.1464130145700675`*^-17, 
      4.852572497872566`*^-13, 3.863392218579291`*^-13, 
      1.3633651776235818`*^-17, 7.778232779576173`*^-13, 
      2.0741630943788776`*^-11, 5.444687407632855`*^-12, 
      2.4889954614611393`*^-11, 9.135189288318186`*^-15, 
      1.7790328207425393`*^-13, 9.123858577875621`*^-14, 
      1.7790328207425393`*^-13, 5.459795192002738`*^-15, 
      1.2711405166529246`*^-13, 6.356332067178278`*^-14, 
      1.6706022543271525`*^-13, 1.2592723611103856`*^-17, 
      1.2653346738390006`*^-17, 1.2594122606984646`*^-17, 
      1.2754540726286122`*^-17, 1.0595527011610362`*^-11, 
      5.8487046585901525`*^-12, 8.790686616778252`*^-12, 
      2.340561725482035`*^-12, 1.1951866164729529`*^-12, 
      3.2369422455791565`*^-12, 3.18714332779511`*^-12, 0.`, 
      6.597450550516379`*^-13, 0.`, 4.838050669081122`*^-12, 
      1.8162771997399537`*^-17, 4.743831121262787`*^-12, 
      6.276099416272394`*^-12, 2.4040717415452828`*^-11, 
      1.3488386753021811`*^-11, 1.9311996885574237`*^-11, 
      1.0809419889200179`*^-12, 3.647803720326494`*^-12, 
      8.978706744058235`*^-13, 4.743831121264441`*^-12, 
      3.836398135656029`*^-12, 2.4397138702953623`*^-12, 
      2.6472995782935565`*^-12, 1.9311996885574237`*^-11, 
      1.0809419889200179`*^-12, 3.6478037203256666`*^-12, 
      8.978706744058235`*^-13, 1.2590528668294985`*^-11, 
      1.2833956433514533`*^-12, 2.055087605477705`*^-12, 
      5.030049728479978`*^-13, 1.272182888476782`*^-12, 
      3.424220925664897`*^-12, 2.528502496643186`*^-10, 
      1.4842039078050867`*^-12, 1.1068068791487135`*^-17, 
      2.0986590407364796`*^-12, 3.3941648643156717`*^-12, 
      1.2779214842247073`*^-17, 3.586128008558862`*^-12, 
      9.56297570931961`*^-11, 2.5102820521843015`*^-11, 
      1.1475570599392401`*^-10, 3.9467142958164596`*^-14, 
      7.693763786360731`*^-13, 3.9455812247711046`*^-13, 
      7.693763786362799`*^-13, 4.7870136750662774`*^-14, 
      1.116688688034506`*^-12, 5.583506388565446`*^-13, 
      1.4676440332315735`*^-12, 1.261772596995439`*^-17, 
      1.2638315229224492`*^-17, 1.2593231850894761`*^-17, 
      1.2689788377756313`*^-17, 6.03946064396063`*^-11, 
      2.6100783129388766`*^-11, 3.049733693690551`*^-11, 
      1.3341178371123963`*^-11, 6.8125227759494354`*^-12, 
      1.8450561010830514`*^-11, 1.8166706419734603`*^-11, 0.`, 
      2.9441795065586414`*^-12, 0.`, 2.159056998011704`*^-11, 
      2.3524135032412796`*^-17, 1.8182057097543697`*^-11, 
      2.5218062374723956`*^-11, 1.1480668212165007`*^-10, 
      6.51548259005046`*^-11, 9.802394611074447`*^-11, 
      2.866229568599636`*^-12, 1.3916531621692905`*^-11, 
      2.170305428244036`*^-12, 1.8182057097543697`*^-11, 
      1.5912621872131294`*^-11, 9.305453092278276`*^-12, 
      1.343011724010955`*^-11, 9.802394611074447`*^-11, 
      2.8662295686004632`*^-12, 1.3916531621696214`*^-11, 
      2.170305428244863`*^-12, 6.298453306196107`*^-11, 
      1.1637221359239102`*^-12, 1.9939741760966107`*^-12, 
      4.651006187628296`*^-13, 8.852657575407775`*^-13, 
      2.8859764736832`*^-12, 1.2833956433514533`*^-12, 
      2.055087605477705`*^-12, 5.030049728479978`*^-13, 
      1.2721828884763684`*^-12, 3.4242209256665515`*^-12, 
      7.073604717928153`*^-11, 1.8355481591815052`*^-13, 
      1.4730460714381452`*^-17, 1.9909573994532865`*^-13, 
      3.218541846645847`*^-14, 1.3684659272834927`*^-17, 
      9.99978652968023`*^-13, 2.6665774277376433`*^-11, 
      6.999775032700939`*^-12, 3.199892661491129`*^-11, 
      3.7553529032921895`*^-15, 7.299647256619369`*^-14, 
      3.7440221928489785`*^-14, 7.299647256616784`*^-14, 
      4.662265641841383`*^-16, 1.0597450349583286`*^-14, 
      5.305020013929812`*^-15, 1.3924120846281417`*^-14, 
      1.2748798100995929`*^-17, 1.2868117018878165`*^-17, 
      1.2606849735094786`*^-17, 1.3166414316754977`*^-17, 
      1.6744140832572096`*^-11, 6.7756938121142575`*^-12, 
      8.56601314038366`*^-12, 3.6987905185343845`*^-12, 
      1.8887502554771055`*^-12, 5.115343767880898`*^-12, 
      5.036646365138414`*^-12, 0.`, 7.643094315690044`*^-13, 0.`, 
      5.604856096877762`*^-12, 1.9887753644391467`*^-17, 
      3.773111877299508`*^-12, 8.369036698752271`*^-12, 
      3.154602785870594`*^-11, 1.7507132797069996`*^-11, 
      1.1257040774609426`*^-11, 3.893369268887309`*^-12, 
      1.639564299455727`*^-11, 2.737497029969094`*^-12, 
      3.773111877297854`*^-12, 5.101648647410001`*^-12, 
      3.2674006410179583`*^-12, 3.657209719887844`*^-12, 
      1.1257040774609426`*^-11, 3.893369268887309`*^-12, 
      1.639564299455727`*^-11, 2.737497029969094`*^-12, 
      1.4769648356766664`*^-11, 9.552847664728005`*^-13, 
      2.3167777375156224`*^-12, 3.525760147136159`*^-13, 
      5.565492454995695`*^-13, 4.76246062006957`*^-12, 
      1.2833956433518669`*^-12, 1.1637221359243238`*^-12, 
      2.055087605477705`*^-12, 1.9939741760966107`*^-12, 
      5.030049728479978`*^-13, 4.651006187628296`*^-13, 
      1.272182888476782`*^-12, 8.852657575411911`*^-13, 
      3.4242209256665515`*^-12, 2.8859764736832`*^-12, 
      1.8225790752890693`*^-17, 1.0976471268728475`*^-11, 
      1.8077426098455047`*^-17, 1.8881117897272003`*^-12, 
      8.45930578776173`*^-14, 1.3633758870231573`*^-17, 
      1.547808138993367`*^-13, 4.127165235571269`*^-12, 
      1.0833901592258216`*^-12, 4.952595764751377`*^-12, 
      3.5508854639182143`*^-14, 6.921897564161544`*^-13, 
      3.5497523928757027`*^-13, 6.921897564161544`*^-13, 
      1.205174279881311`*^-15, 2.783956371585065`*^-14, 
      1.3926076697065109`*^-14, 3.658518412765815`*^-14, 
      1.2600199600439004`*^-17, 1.299642865496047`*^-17, 
      1.2606084191595792`*^-17, 1.313962034168181`*^-17, 
      1.8784688934527903`*^-12, 1.5797420754772088`*^-12, 
      2.22316028630751`*^-12, 4.1496358718233006`*^-13, 
      2.1190246074416793`*^-13, 5.738809904814151`*^-13, 
      5.65052245854096`*^-13, 0.`, 1.7820607567638056`*^-13, 0.`, 
      1.306764820331647`*^-12, 1.0831022343737338`*^-17, 
      8.035012166456962`*^-13, 1.8082434285577954`*^-12, 
      5.366827927958573`*^-12, 3.2908177429945083`*^-12, 
      2.5771267386398857`*^-12, 8.099636572129681`*^-13, 
      1.979762711464747`*^-12, 8.794695771002078`*^-13, 
      8.035012166454894`*^-13, 1.1124685080269962`*^-12, 
      6.957875102097962`*^-13, 5.815530310471105`*^-13, 
      2.5771267386390585`*^-12, 8.099636572133817`*^-13, 
      1.979762711464747`*^-12, 8.794695771002078`*^-13, 
      2.411360755571643`*^-12, 7.37312017432458`*^-13, 
      1.8846286069101253`*^-12, 7.437029752294637`*^-13, 
      2.9723962924313536`*^-12, 3.364905879319247`*^-12, 
      1.2833956433518669`*^-12, 1.1637221359243238`*^-12, 
      9.552847664728005`*^-13, 0.`, 4.049049191896973`*^-12, 
      2.3167777375156224`*^-12, 4.76246062006957`*^-12, 
      1.5662373357227446`*^-17, 1.5221816429929542`*^-11, 
      1.5725260846289217`*^-17, 4.2623986281826656`*^-13, 
      5.978738980804696`*^-14, 1.3357799103011001`*^-17, 
      2.146401113624809`*^-13, 5.723413167925917`*^-12, 
      1.5024052414679861`*^-12, 6.868093283573646`*^-12, 
      8.025662413303391`*^-15, 1.5626750801139597`*^-13, 
      8.014331702857595`*^-14, 1.5626750801139597`*^-13, 
      8.554143601008301`*^-16, 1.9678498920964414`*^-14, 
      9.84554429961876`*^-15, 2.5859213254379836`*^-14, 
      1.2592723604616264`*^-17, 1.26533465788066`*^-17, 
      1.2594122597000823`*^-17, 1.2754540313124956`*^-17, 
      3.188935857611944`*^-12, 1.401437568339448`*^-12, 
      2.4598467437505414`*^-12, 7.044457395646681`*^-13, 
      3.597231343011615`*^-13, 9.742286480318861`*^-13, 
      9.59240708672401`*^-13, 0.`, 1.5809332727125024`*^-13, 0.`, 
      1.159271332027427`*^-12, 4.845480800807509`*^-18, 
      8.981037596321278`*^-13, 2.2345537948408876`*^-12, 
      6.804814742241486`*^-12, 3.673747357232187`*^-12, 
      2.4503494272405342`*^-12, 1.6060104952741514`*^-12, 
      2.7484799990856213`*^-12, 1.8094816661661023`*^-12, 
      8.981037596321278`*^-13, 1.4516487819744168`*^-12, 
      7.829176025442269`*^-13, 7.412872940406472`*^-13, 
      2.4503494272413614`*^-12, 1.6060104952737378`*^-12, 
      2.7484799990856213`*^-12, 1.8094816661661023`*^-12, 
      1.864278280743014`*^-12, 1.7572642518344909`*^-12, 
      5.758126065224592`*^-12, 2.8971897932691615`*^-12, 
      4.211401281741377`*^-12, 1.8144973548862373`*^-11, 
      1.887871989786091`*^-17, 1.886967823217958`*^-17, 
      1.8881111478146088`*^-17, 7.12733736053604`*^-12, 
      8.862492422801712`*^-12, 1.2779222217506469`*^-17, 
      1.285629449518804`*^-17, 1.9681125370997488`*^-17, 
      1.4451709676099783`*^-17, 2.109927189593136`*^-17, 
      1.3400629537045715`*^-13, 2.6128898506766375`*^-12, 
      1.3399496465998034`*^-12, 2.6128898506758103`*^-12, 
      1.2497355532531654`*^-13, 2.915768454777194`*^-12, 
      1.4578905222272682`*^-12, 3.832148869521869`*^-12, 
      1.2617727025731115`*^-17, 1.2638317081643447`*^-17, 
      1.2593231961357376`*^-17, 1.2689792218612776`*^-17, 
      3.15801787193693`*^-12, 1.950917156030985`*^-12, 
      1.3361248916152718`*^-12, 6.976159565295667`*^-13, 
      3.5623558551717656`*^-13, 9.647832034080464`*^-13, 
      9.499405785820501`*^-13, 0.`, 2.2007462476274834`*^-13, 0.`, 
      1.6138008469646316`*^-12, 1.3052701653787715`*^-17, 
      1.8577070443764367`*^-17, 2.0133846513625623`*^-12, 
      6.428489360790848`*^-12, 5.30793035991201`*^-12, 
      2.395644385924532`*^-12, 1.768001709062543`*^-12, 
      2.2648684451621806`*^-12, 2.92460411371068`*^-12, 
      1.857707095661862`*^-17, 7.666809254768473`*^-13, 
      1.2467163155642983`*^-12, 8.352525866056711`*^-13, 
      2.395644385924532`*^-12, 1.768001709062543`*^-12, 
      2.2648684451613534`*^-12, 2.9246041137115072`*^-12, 
      2.3833388358786722`*^-12, 9.51908980111979`*^-12, 
      2.6364344990563647`*^-11, 1.1253952209032622`*^-11, 
      1.7671412853457103`*^-11, 5.824014360405186`*^-11}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(myValue\  = \ c . mySol\), "\[IndentingNewLine]", 
    \(myTolerance\  = Norm[A . mySol\  - \ b]\)}], "Input"],

Cell[BoxData[
    \(1.7229822993901713`*^-8\)], "Output"],

Cell[BoxData[
    \(5.658548468537552`*^-16\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Export", "[", 
      RowBox[{
        RowBox[{"rootDir", " ", "<>", 
          StyleBox["\"\<value\>\"",
            "MR"], 
          StyleBox[" ",
            "MR"], 
          StyleBox["<>",
            "MR"], "problemId", "<>", " ", 
          StyleBox["\"\<.txt\>\"",
            "MR"]}], 
        StyleBox[",",
          "MR"], " ", Cell[BoxData[
            FormBox[
              StyleBox["myValue",
                "MR"], TraditionalForm]]], ",", " ", "\"\<Lines\>\""}], 
      "]"}]], "Input"],

Cell[BoxData[
    \("C:\\DATADIR\\Programmazione\\java\\joptimizer\\src\\test\\resources\\\
lp\\presolving\\value8.txt"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Export", "[", 
      RowBox[{
        RowBox[{"rootDir", " ", "<>", 
          StyleBox["\"\<sol\>\"",
            "MR"], 
          StyleBox[" ",
            "MR"], 
          StyleBox["<>",
            "MR"], "problemId", "<>", " ", 
          StyleBox["\"\<.txt\>\"",
            "MR"]}], 
        StyleBox[",",
          "MR"], " ", Cell[BoxData[
            FormBox[
              StyleBox["mySol",
                "MR"], TraditionalForm]]], ",", " ", "\"\<Lines\>\""}], 
      "]"}]], "Input"],

Cell[BoxData[
    \("C:\\DATADIR\\Programmazione\\java\\joptimizer\\src\\test\\resources\\\
lp\\presolving\\sol8.txt"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Export", "[", 
      RowBox[{
        RowBox[{"rootDir", " ", "<>", 
          StyleBox["\"\<tolerance\>\"",
            "MR"], 
          StyleBox[" ",
            "MR"], 
          StyleBox["<>",
            "MR"], "problemId", "<>", " ", 
          StyleBox["\"\<.txt\>\"",
            "MR"]}], 
        StyleBox[",",
          "MR"], " ", Cell[BoxData[
            FormBox[
              StyleBox["myTolerance",
                "MR"], TraditionalForm]]], ",", " ", "\"\<Lines\>\""}], 
      "]"}]], "Input"],

Cell[BoxData[
    \("C:\\DATADIR\\Programmazione\\java\\joptimizer\\src\\test\\resources\\\
lp\\presolving\\tolerance8.txt"\)], "Output"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 919}},
WindowSize->{1270, 880},
WindowMargins->{{0, Automatic}, {Automatic, 0}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 95, 5, 138, "Subtitle"],
Cell[1874, 60, 208, 4, 61, "Input"],

Cell[CellGroupData[{
Cell[2107, 68, 4364, 144, 243, "Input"],
Cell[6474, 214, 37, 1, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6548, 220, 91, 1, 35, "Input"],
Cell[6642, 223, 12840, 213, 1560, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19519, 441, 131, 2, 61, "Input"],
Cell[19653, 445, 57, 1, 35, "Output"],
Cell[19713, 448, 57, 1, 35, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19807, 454, 530, 17, 35, "Input"],
Cell[20340, 473, 133, 2, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20510, 480, 526, 17, 35, "Input"],
Cell[21039, 499, 131, 2, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[21207, 506, 538, 17, 35, "Input"],
Cell[21748, 525, 137, 2, 34, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

